local p = ...
local fade_out_speed = 0.2
local fade_out_pause = 0.1
local off_wait = 0.75

local t = Def.ActorFrame{
	OffCommand=function(self)
		self:sleep(fade_out_pause):decelerate(fade_out_speed):diffusealpha(0)
	end;
};
-- A very useful table...
local NoteTable = getenv("perColJudgeData")

local eval_part_offs = string.find(p, "P1") and -310 or 310
local score_parts_offs = string.find(p, "P1") and -100 or 100

local function GetColumnPositions()
	return {
		dance = {
			Dance_Single 		= { "Left", "Down", "Up", "Right" },
			-- This will be looped by the style check.
			Dance_Double 		= { "Left", "Down", "Up", "Right" },
			Dance_Couple		= { "Left", "Down", "Up", "Right" },
			--
			Dance_Solo 			= { "Left", "UpLeft", "Down", "Up", "UpRight", "Right" },
			Dance_Threepanel 	= { "Left", "Down", "Right" },
		},
		pump = {
			Pump_Single 		= {"DownLeft", "UpLeft", "Center", "UpRight", "DownRight"},
			-- This will be looped by the style check.
			Pump_Double 		= {"DownLeft", "UpLeft", "Center", "UpRight", "DownRight"},
			Pump_Halfdouble 	= { "Center", "UpRight", "DownRight", "DownLeft", "UpLeft", "Center" },
		},
		techno = {
			Techno_Single4 		= { "Left", "Down", "Up", "Right" },
			Techno_Single8 		= { "DownLeft", "Left", "UpLeft", "Down", "Up", "UpRight", "Right", "DownRight" },
		},
		kb7 = {
			Kb7_Single 			= { "Key1", "Key2", "Key3", "Key4", "Key5", "Key6", "Key7" },
		},
		para = {
			Para_Single 		= { "Left", "UpLeft", "Up", "UpRight", "Right" },
		},
		smx = {
			Smx_Single 			= { "Left", "Down", "Center", "Up", "Right" },
			-- These two will be looped by the style check.
			Smx_Double6 		= { "Left", "Center", "Right" },
			Smx_Double10 		= { "Left", "Down", "Center", "Up", "Right" },
			Smx_Wide 			= { "Left", "Center", "Right", "Left", "Center", "Right" },
		},
	};
end

-- Step counts.
t[#t+1] = Def.BitmapText {
    Font = "_plex sans Bold 24px",
    InitCommand=function(self)
        self:zoom(1):xy(_screen.cx +(eval_part_offs),_screen.cy-165+70):maxwidth(260):horizalign(center)
        self:diffuse(Color.White):diffusebottomedge(ColorLightTone(PlayerCompColor(p)))
    end;
    Text=THEME:GetString("ScreenEvaluation","ColumnSc");
};

-- offset and spacing for the columns
local ColumnSpacing = {
	[03] = { eval_part_offs-120, 60 },
	[04] = { eval_part_offs-150, 60 },
	[05] = { eval_part_offs-150, 50 },
	[06] = { eval_part_offs-140, 40 },
	[07] = { eval_part_offs-168, 42 },
	[08] = { eval_part_offs-180, 40 },
	[10] = { eval_part_offs-175, 32 },
};
local Name,Length = LoadModule("Options.SmartTapNoteScore.lua")()
table.sort(Name)
Name[#Name+1] = "Miss"
Length = Length + 1
for i=1,GAMESTATE:GetCurrentStyle():ColumnsPerPlayer() do
	for ind,val in ipairs( Name ) do
		local cur_line = "JudgmentLine_" .. val
		t[#t+1] = Def.BitmapText{
			Font="_plex condensed semibold 24px",
			OnCommand=function(self)
				local col = GAMESTATE:GetCurrentStyle():ColumnsPerPlayer();
				self:x( _screen.cx + ColumnSpacing[col][1] + ColumnSpacing[col][2]*i )
				:y( SCREEN_CENTER_Y-30+((44-(Length*2.6))*ind)):zoom(1.5-(Length*0.1) )
				:settext( NoteTable[p][i][val] )
				self:diffuse( BoostColor( JudgmentLineToColor(cur_line), 1 ) )
				self:diffusealpha(0):sleep(0.1 * ind):decelerate(0.6):diffusealpha(0.86)
			end;
		};
	end
end

-- Noteskin Operation
local pstate = GAMESTATE:GetPlayerState(p)
local noteskin = pstate:GetPlayerOptions('ModsLevel_Song'):NoteSkin():lower()
local style = ToEnumShortString(GAMESTATE:GetCurrentStyle():GetStyleType())
local gamemode = GAMESTATE:GetCurrentGame():GetName()

-- Column Positions
-- Needed for the noteskin display
local columnpos = GetColumnPositions()

-- Check for styles
local sttype = ToEnumShortString(GAMESTATE:GetCurrentSteps(p):GetStepsType())
local num_columns = #columnpos[gamemode][sttype]

-- Specific check because Half-Double has different
-- columns regardless of side, so duplicating it won't work.
if style == "OnePlayerTwoSides" and sttype ~= "Pump_Halfdouble" then
	for i=1,num_columns do
		table.insert(columnpos[gamemode][sttype], columnpos[gamemode][sttype][i])
	end
end

-- Print the noteskin columns
for i,col in ipairs( columnpos[gamemode][sttype] ) do
	t[#t+1] = NOTESKIN:LoadActorForNoteSkin( col, "Tap Note", noteskin)..{
		OnCommand=function(self)
			local col = GAMESTATE:GetCurrentStyle():ColumnsPerPlayer();
			self:xy( _screen.cx+ColumnSpacing[col][1] + ColumnSpacing[col][2]*i, _screen.cy-50):zoom(0.6)
		end;
	};
end

t[#t+1] = Def.GraphDisplay{
	InitCommand=function(self) self:vertalign(bottom):x(_screen.cx + (eval_part_offs)):y(_screen.cy+196+70) end;
	BeginCommand=function(self)
		self:Load("GraphDisplayTransp")
		local playerStageStats = STATSMAN:GetCurStageStats():GetPlayerStageStats(p)
		local stageStats = STATSMAN:GetCurStageStats()
		self:Set(stageStats, playerStageStats)
	end,
	OnCommand=function(self)
		self:zoomy(0):sleep(1.2):decelerate(0.4):zoomy(1)
	end;
	OffCommand=function(self)
		self:sleep(fade_out_pause):decelerate(fade_out_speed):diffusealpha(0)
	end;
	PageUpdatedMessageCommand=function(self)
		local PageInd = getenv("PageIndex")
		self:finishtweening():decelerate(0.2)
		:diffusealpha( PageInd[p] == 2 and 1 or 0 )
	end;
	ScatterZoomMessageCommand=function(s,param)
		local PageInd = getenv("PageIndex")
		s:diffusealpha( (PageInd[p] == 2 and param.Zoom > 1) and 0.3 or 1)
	end
};

t[#t+1] = Def.BitmapText {
    Font = "_plex sans Bold 24px",
    InitCommand=function(self)
        self:zoom(1):xy(_screen.cx +(eval_part_offs)-160 ,_screen.cy-165+340):maxwidth(260):horizalign(left):zoom(0.6)
        self:diffuse(Color.White):diffusebottomedge(ColorLightTone(PlayerCompColor(p))):diffusealpha(0.5)
    end;
	ScatterZoomMessageCommand=function(s,param)
		if param.Player == p then
			s:settext( string.format( THEME:GetString("ScreenEvaluation","CurrentZoom"), param.Zoom ) )
		end
	end
};

t[#t+1] = Def.BitmapText {
	Font = "_plex sans Bold 24px",
    InitCommand=function(self)
        self:zoom(1):xy(_screen.cx +(eval_part_offs)+160 ,_screen.cy-165+340):maxwidth(340):horizalign(right):zoom(0.6)
        self:diffuse(Color.White):diffusebottomedge(ColorLightTone(PlayerCompColor(p))):diffusealpha(0.5)
    end;
	ScatterZoomMessageCommand=function(s,param)
		if param.Player == p then
			local touse = param.Zoom > 1 and "ZoomHelp2" or "ZoomHelp1"
			s:settext( THEME:GetString("ScreenEvaluation",touse) )
		end
	end
};

t[#t+1] = Def.ActorFrameTexture{
	InitCommand=function(s)
		s:SetWidth( 333 ):SetHeight( 78 ):EnableAlphaBuffer(true):SetTextureName("Scatterplot"..p):Create()
	end,

	Def.ActorFrame{
		OnCommand=function(s)
			s:x( 333/2 )
		end,
		Def.Quad{
			OnCommand=function(s)
				s:zoomto(333,78):diffusetopedge( color("#FF8D47") ):diffusebottomedge( color("#FFC447") ):diffusealpha(0.4)
				:xy(0,78/2):MaskDest()
			end,
			ScatterZoomMessageCommand=function(s,param)
				if param.Player == p then
					s:zoomto( scale( param.Zoom, 1, 10, 333, 333/10 ) ,78 )
					:x( scale(param.Xpos, 1, param.MaxSegment, 0, (333/param.MaxSegment)) )
				end
			end
		};
	};

	Def.ActorFrame{
		InitCommand=function(self) self:xy(333/2,0) end;
		LoadActor("Scatter.lua",p)..{
			OnCommand=function(self)
				self:vertalign(bottom)
			end;
			OffCommand=function(self)
				self:sleep(fade_out_pause):decelerate(fade_out_speed):diffusealpha(0)
			end;
		};
	};
}

t[#t+1] = Def.Sprite{
	Texture="Scatterplot"..p,
	OnCommand=function(s)
		s:xy( _screen.cx + (eval_part_offs), _screen.cy+188+(78/2) )
	end,
}

return t;
